/*  This file is part of Lilisp.
 *  Copyright (C) 2008  Julio Cesar Serrano Ortuno
 * 
 *  Lilisp is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Lilisp is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Lilisp.  If not, see <http://www.gnu.org/licenses/>.
 */

package julk.net.w3s.language;
/* 
 * Ideas para esta clase:
 * 
 * Cada funcin puede ser definida como un nuevo programa.
 * Para ello, cada programa debe devolver un valor que,
 * en el caso de las funciones ser el valor de retorno.
 * 
 * Para llamar a una funcion, se pasan los parmetros como
 * variables locales y se instancia un objeto Program
 * con el cdigo de la funcin, tras lo cual se llama al 
 * mtodo execute.
 * 
 * Al leerse el cdigo se van creando las instrucciones una
 * a una, almacenndose en un vector. Cada instruccin se
 * crear con el nombre del archivo de donde procede y su
 * nmero de lnea. En caso de error deben mencionarse estos
 * datos y el cdigo que provoc el error.
 * 
 * Sera interesante que las funciones pudiesen devolver 
 * cdigo fuente para su posterior procesamiento. Esto
 * evitara la necesidad de preprocesar los fuentes para
 * cargar fragmentos de cdigo de otros archivos.
 */
import java.io.BufferedReader;
//import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Stack;

import julk.net.w3s.annotations.ClassInfo;
import julk.net.w3s.language.bit.*;

@ClassInfo (
		id = 3,
		name = "Program",
		creationDate = "March 10, 2008",
		description = "Main interpreter program"
)
public class Program {
	
	//private String code;
	private References references;
	private Variables global;
	private Types types;
	private Stack<Variables> temp;
	private Functions functions;
	private Macros macros;
	private RequestInput input;
	//private Vector<LanguageBit> bits;
	private int line;
	
	//Debug mode (true = ON/ false = OFF)
	//public static final boolean DEBUG = true;
	private boolean DEBUG = false;
	
	public Program() {
		references = new References();
		global = new Variables(references);
		global.setNamespace("Global");
		types = new Types("Global", references);
		temp = new Stack<Variables>();
		temp.push(global);
		functions = new Functions(this);
		macros = new Macros(this);
		//input = new RequestInput();
		//bits = new Vector<LanguageBit>();
		//String code = Program.loadCode(filename);
		//System.out.println(code);
		//code = preprocess(code);
		//code = code;
		//initBits();		
	}
	
	/*public static Program loadProgram(String filename) throws Exception {
		Program p = new Program();
		p.global = new Variables();
		p.global.setNamespace("Global");
		p.temp = new Stack<Variables>();
		p.temp.push(p.global);
		p.functions = new Functions(p);
		p.bits = new Vector<LanguageBit>();
		String code = Program.loadCode(filename);
		System.out.println(code);
		code = preprocess(code);
		p.code = code;
		p.initBits();
		return p;
	}*/
	
	/*public void initBits() {
		bits.addElement(new Operators());
		bits.addElement(new FunctionsDefinition());
		bits.addElement(functions);
		bits.addElement(new ControlStructures());
		bits.addElement(new IncludeControl());
		bits.addElement(new IOControl());
		bits.addElement(new DataTypes());
		bits.addElement(new StringOperator());
		bits.addElement(new ObjectsHandler());
	}*/
	
	/*public LanguageBit getResponsibleBit(String cmd) {
		Iterator<LanguageBit> i = bits.iterator();
		while(i.hasNext()) {
			LanguageBit bit = i.next();
			if (bit.belongs(cmd)) {
				return bit;
			}
		}
		return null;
	}*/
	/**
	 * @deprecated
	 */
	private static String removeLineComments(String line) {
		String res = "";
		char c;
		for (int i = 0; i < line.length(); i++) {
			if ((c = line.charAt(i)) != ';') {
				res += c;
			} else {
				break;
			}
		}
		return res;
	}
	
	/**
	 * @deprecated
	 * @param filename
	 * @return
	 * @throws Exception
	 */
	public static String loadCode(String filename) throws Exception {
		BufferedReader bin = new BufferedReader(new FileReader(filename));
		String content = "", line;
		while((line = bin.readLine()) != null) {
			content+=removeLineComments(line);
		}
		bin.close();
		return prepareCode(content);
	}
	
	/**
	 * @deprecated
	 * @param code
	 * @return
	 */
	public static String preprocess(String code) {
		String pcode = "", file, content;
		do {
			try {
				Instruction instr = new Instruction(readInstruction(code));
				//if (instr.getCode().length() < code.length()) {
				code = code.substring(instr.getCode().length());
				//} else {
				//	code = "";
				//}
				/*StringTokenizer st = new StringTokenizer(instr, "( )");
				if (st.countTokens() == 2) {
					cmd = st.nextToken();
					file = st.nextToken();
					if (cmd.equalsIgnoreCase("include")) {
						try {
							content = loadCode(file);
						} catch (Exception e) {
						}
					}
				}*/
				if (instr.getCmd().equalsIgnoreCase("include") &&
					instr.countArguments() == 2) {
					file = instr.getArgs().argumentAt(1).getValue();
					try {
						content = loadCode(file);
						content = preprocess(content);
						pcode += content;
					} catch (Exception e) {
						e.printStackTrace();
					}
				} else {
					pcode += instr.getCode();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		} while (code.length() > 0);
		return pcode;
	}
	
	/*public void execute() {
		execute(this.code);
	}*/
	
	/**
	 * This method uses readInstruction to load the 
	 * instructions from a String.
	 * Modern method uses a List to load the instructions.
	 * @deprecated
	 */
	public String executeOld(String code, Instruction orig, boolean loop) 
	throws Exception {
		if (code == null || code.length() == 0)
			return "";
		if (code.charAt(0) != '(') return code;
		int instNumber = 1;
		//String xcode = new String(prepareCode(code));
		//String xcode = new String(code);
		String xcode = code.trim();
		String result = "", icode;
		do {
			icode = readInstruction(xcode);
 			if (icode == null || icode.length() == 0) {
				break;
			}
			Instruction instr = new Instruction(orig.getFile(), orig.getLine()+instNumber, icode);
			xcode = xcode.substring(icode.length());
			//Line adjust
			//instr.setLine(instr.getLine()+instNumber);
			if (DEBUG) {
				System.out.println(instr.getFile()+"/"+instr.getLine()+":"+instr.getCode());
			}			
			result = evaluate(instr);
			instNumber++;			
			if (result.equalsIgnoreCase("error")) {
				return "error";
			} /*else if (result.equalsIgnoreCase("return")) {
				String returnVal = getReturnValue(); 
				if (loop) 
					result = "return";
				else 
					result = returnVal; 
				break;
			}*/
			if (DEBUG) {
				System.out.println(result);
			}
		} while (xcode.length() > 0);
		return result;
	}

	/**
	 * This method is called to execute functions and flow
	 * control code.
	 * 
	 * @param code
	 * @param orig
	 * @param loop
	 * @return
	 * @throws Exception
	 */
	public String execute (String code, Instruction orig, boolean loop) 
	throws Exception {
		if (code == null || code.length() == 0)
			return "";
		String xcode = code.trim();
		if (!Type.list.isValidValue(xcode)) 
			return code;
		//if (code.charAt(0) != '(') return code;
		int instNumber = 1;
		String result = "", icode;
		List lcode = new List(xcode);
		lcode.makeMultiList();
		while ((icode = lcode.head()) != null) {
			
 			if (icode == null || icode.length() == 0) {
				break;
			}
 			if (icode.trim().length() == 0)
 				continue;
			Instruction instr = new Instruction(orig.getFile(), orig.getLine()+instNumber, icode);
			if (DEBUG) {
				System.out.println(instr.getFile()+"/"+instr.getLine()+":"+instr.getCode());
			}			
			result = evaluate(instr);
			instNumber++;			
			if (result.equalsIgnoreCase("error")) {
				return "error";
			}
			if (DEBUG) {
				System.out.println(result);
			}
			lcode = lcode.tail();
		}
		return result;
	}
	
	/*public String getReturnValue() {
		Variable var = getTemp().get("@this:return");
		return var != null ? var.getValue().toString() : "";
	}*/
	
	/**
	 * Nueva funcion de ejecucin de archivos
	 * @param filename 
	 * @return
	 */
	public boolean executeFile(String filename) {
		//Testing filename
		if (filename == null || filename.length() == 0) {
			return false;
		}
		
		String retval = "";
		//Read instructions and validate it
		try {
			BufferedReader in = new BufferedReader(new FileReader(filename));
			String code = "";
			int c;
			while((c = in.read()) != -1) {
				code += (char) c;
			}
			in.close();
			List lcode = new List(code);
			String head = "";
			Instruction inst;
			String result;
			while ((head = lcode.head()) != null) {
				inst = new Instruction(filename,lcode.iLine,head);
				if (DEBUG) {
					System.out.println(inst.getFile()+"/"+inst.getLine()+":"+inst.getCode());
				}
				result = evaluate(inst);
				if (result.equalsIgnoreCase("error")) {
					break;
				} /*else if (result.equalsIgnoreCase("return")) {
					result = getReturnValue();
					break;
				}*/
				if (DEBUG) {
					System.out.println("Result: "+result);
				}
				lcode = lcode.tail();
			}
			in.close();
		} catch (ReturnException re) {
			retval = re.getMessage();
			/*int pos = retval.indexOf(" ");
			if (pos == -1) {
				retval = "";
			} else {
				retval  =  retval.substring(pos+1);*/
		
			if (References.isReference(retval)) {
				Reference ref = references.get(retval);
				ref.type = Type.string;
				retval = ref.value.toString();				
			}
			//}
		} catch (CodeException ce) {
			return false;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		if (DEBUG) {
			//System.out.println("File "+filename+" read. "+getReturnValue());
			System.out.println("File "+filename+" read. "+retval);
		}
		return true;
	}
	
	/**
	 * This functions loads and executes the code from
	 * a file using readInstruction (improved version)
	 * @deprecated
	 * @param filename 
	 * @return
	 */
	public boolean executeFileOld(String filename) {
		//Testing filename
		if (filename == null || filename.length() == 0) {
			return false;
		}
		
		String retval = "";
		//Read instructions and validate it
		try {
			BufferedReader in = new BufferedReader(new FileReader(filename));
		
			Instruction inst;
			String result;
			while ((inst = readInstruction(filename, in)) != null) {
				if (DEBUG) {
					System.out.println(inst.getFile()+"/"+inst.getLine()+":"+inst.getCode());
				}
				result = evaluate(inst);
				if (result.equalsIgnoreCase("error")) {
					break;
				} /*else if (result.equalsIgnoreCase("return")) {
					result = getReturnValue();
					break;
				}*/
				if (DEBUG) {
					System.out.println("Result: "+result);
				}
			}
			in.close();
		} catch (ReturnException re) {
			int pos = retval.indexOf(" ");
			if (pos == -1) retval = "";
			retval =  retval.substring(pos+1);

			retval = re.getMessage();
			Reference ref = references.get(retval);
			retval = ref.value.toString();
			/*int pos = retval.indexOf(" ");
			if (pos == -1) return "";
			return retval.substring(pos+1);*/
		} catch (CodeException ce) {
			return false;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		if (DEBUG) {
			//System.out.println("File "+filename+" read. "+getReturnValue());
			System.out.println("File "+filename+" read. "+retval);
		}
		return true;
	}

	/**
	 * This method evaluates a instruction recursively.
	 * It have also into account macros and functions.
	 * @param instr
	 * @return
	 * @throws Exception
	 */
	public String evaluate(Instruction instr) throws Exception {
		Arguments args = instr.getArgs();
		String code;
		
		//Is it a macro?
		Macro m; 
		if ((m = macros.get(instr.getCmd())) != null) {
			Instruction inst = new Instruction(m.call(instr));
			inst.setFile(instr.getFile());
			inst.setLine(instr.getLine());
			
			return evaluate(inst);
		}
		//LanguageBit bit = getResponsibleBit(instr.getCmd());
		BitCommand bit = Bit.locateBit(instr);
		/*if (bit == null) {
			return "";
		}*/
		if (bit == null || !bit.isLazy()) {
		//Evaluar expresiones de los argumentos
		/*if (!instr.getCmd().equalsIgnoreCase("defun") &&
			!instr.getCmd().equalsIgnoreCase("if") &&
			!instr.getCmd().equalsIgnoreCase("for") &&
			!instr.getCmd().equalsIgnoreCase("while") &&
			!instr.getCmd().equalsIgnoreCase("do")) {*/
			for (int i = 0; i < args.countArgs(); i++) {
				Argument arg = args.argumentAt(i); 
				if (arg.isQuoted())
					continue;
				code = arg.getValue();
				if (code.length() > 0 && code.charAt(0) == '(') {
					Instruction aux = new Instruction(instr.getFile(), instr.getLine(), code);
					String result = evaluate(aux);
					if (result.equalsIgnoreCase("error")) {
						throw new Exception("Error: File "+instr.getFile()+
								            " line "+instr.getLine()+
								            " evaluation error.");
					}
					arg = new Argument(result);
					args.setArgumentAt(arg, i);
					if (i == 0) {	
						if (References.isReference(result)) {
							Reference ref = references.get(result);
							instr.setCmd(ref.value.toString());
						} else {
							instr.setCmd(result);
						}
						instr.getArgs().argumentAt(i).setValue(instr.getCmd());
						bit = Bit.locateBit(instr);
					}
				}
			}
		}
		//Ejecutar esta orden
		if (bit != null)
			return Bit.evaluateBit(this, getTemp(), instr);
		else {
			return functions.evaluate(this, getTemp(), instr);
		}
		/*bit = getResponsibleBit(instr.getCmd());
		if (bit == null) {
			return "";
		} else {
			return bit.execute(this, getTemp(), instr);
		}*/
	}

	/**
	 * This method read a String and returns the first
	 * instruction os the list.
	 * The String has to be preprocessed removing
	 * comments and spaces to work well.
	 * @deprecated
	 * @param code
	 * @return
	 */
	public static String readInstruction(String code) {
		if (code == null || code.length() == 0) {
			return null;
		}
		int pos = code.indexOf("(");
		if (pos == -1) 
			return null;
		//code = code.substring(pos);
		
		boolean quotes = false;
		int blocks = 1, endBlock = 0;
		char c;
		for (int i = pos+1; i < code.length(); i++) {
			c = code.charAt(i);
			if (c == '\"') quotes = !quotes;
			if (!quotes && c == ' ' && blocks == 0)
				continue;
			if (c == '(')
				blocks++;
			else if (c == ')') 
				blocks--;
			if (blocks == 0) {
				endBlock = i;
				break;
			}
		}
		
		String result = code.substring(0,endBlock+1);
		return result;
	}
	
	/**
	 * This is the improved version of 
	 * readInstruction. It takes instructions
	 * from a file and calculates line numbers.
	 * Takes into account comments and spaces.
	 * 
	 * It's now deprecated because I wanted to be
	 * logic. Now a program is a list (of instructions),
	 * so, why not using a list to read the instructions?
	 * So I did. now executeFile loads instructions using
	 * a list. Though, this version of readInstruction
	 * works pretty well. If you're having problems
	 * with the list, and you want a quick solution,
	 * reuse this method.
	 * 
	 * By now I'll deprecate it to prevent using it.
	 * 
	 * @deprecated
	 * @param file
	 * @param in
	 * @return
	 */
	public Instruction readInstruction(String file, BufferedReader in) {
		
		//No hay buffer, no hay instrucciones
		if (in == null) return null;
		
		//Datos para procesar caracteres
		boolean quotes = false; //Entre comillas?
		boolean comment = false; //Es un comentario?
		char commentInit = ' ';
		String code = ""; //Contendr el cdigo resultante;
		int blocks = 0;
		boolean hasInstruction = false;
		boolean hasLine = false;
		char c,     //actual character 
		     pc = 'X';    //previous character
		int myLine = 0;  //Current instruction's line

		//If the line number 
		if (line == 0) {
			line = 1;
		}
		int ic;
		try {
			while ((ic = in.read()) != -1) {
				if (code.trim().length() > 0 && !hasLine) {
					/*if (code.charAt(0) == ';' ||
						code.charAt(0) == '#') {
						if (comment) continue;
						commentInit = code.charAt(0);
						comment = true;
						code = "";
						continue;
					}*/
					myLine = line;
					hasLine = true;
				}
				c = (char)ic;
				if (blocks < 0) {
					System.out.println("Code error: Spare parenthesys on file "+file+" line "+myLine);
					return null;
				}
				if (c == '\n')
					line++;
				if (c == '\"' && !comment) {
					quotes = !quotes;
					code += c;
					pc = c;
					continue;
				}
				if (quotes) {
					code += c;
					continue;
				}
				if (c == '(' && !comment && !quotes)  {
					hasInstruction = true;
					blocks++;
				}
				if (c == ')' && !comment && !quotes) blocks--;
				if (c == '#')  {
					if (!comment) commentInit = '#';
					comment = !comment;
					continue;
				}
				if (c == ';' && !comment) {
					commentInit = ';';
					comment = !comment;
				}
				if (c == '\n' && comment && commentInit == ';') {
					comment = false;
				}
				if (comment) 
					continue;
				if (c == '\t') c = ' ';
				if (c == 0x0d || c == 0x0a) { // || c == '\t') {
					continue;
				}
				if (c == ' ' && (pc == ' ' || pc == '(')) {
					pc = c;
					continue;
				}
				code += c;
				pc = c;
				if (blocks == 0 && hasInstruction) {
					break;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		if (ic == -1) {
			if (blocks > 0 && hasInstruction ) {
				System.out.println("Code error: end of file reached, parenthesys missing. Error located in file "+file+" from line "+myLine+" on.");
				return null;
			} else if (!hasInstruction){
				return null;
			}
		}
		//return code;
		Instruction inst = null;
		try {
			inst = new Instruction(file,myLine,code.trim());
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return inst;
	}
	
	/**
	 * This method isn't used anymore. It
	 * used to test a character to see if it was valid or not.
	 * Invalid characters are \r, \n, space and \t.
	 * 
	 * @deprecated
	 * @param c
	 * @return
	 */
	private static boolean invalid(char c) {
		char inv[] = {0x0a, 0x0d, 32, '\t'};
		
		for (int i = 0; i < inv.length; i++) {
			if (c == inv[i])
				return true;
		}
		return false;
	}
	
	/**
	 * This method is old as many others here.
	 * In the first versions it used to fix
	 * the code to be readable for the rest of the methods.
	 * 
	 * Now I have set some more powerful dealing methods
	 * that are able to remove comments and spaces on the fly.
	 * 
	 * @deprecated
	 * @param code
	 * @return
	 */
	public static String prepareCode(String code) {
		//if (code == code) return code;
		boolean quotes = false;
		boolean comment = false; 
		//char commentsInit = '#';
		String pcode = "";
		int i, blocks = 0;
		
		char c, car;
		
		for (i = 0; i < code.length(); i++) {
			car = code.charAt(i);
			if (car == '\"') quotes = !quotes;
			if (car == '(' && !comment && !quotes) blocks++;
			if (car == ')' && !comment && !quotes) blocks--;
			if (car == '#') comment = !comment;
			if (comment) 
				continue;
			if (!comment && code.charAt(i) == '#')
				continue;
			if (!quotes) {
				if (!invalid(code.charAt(i))) {
					pcode += code.charAt(i);
				} else {
					if (i == 0) {
						while((i+1) < code.length() && invalid(code.charAt(i+1))) {
							i++;
						}
					} else {
						c = code.charAt(i-1);
						while((i+1) < code.length() && invalid(code.charAt(i+1))) {
							i++;
						}
						if (code.charAt(i+1) == ')') {
							continue;
						}
						if ((c != '#' && c != '(') && (c != ')' || blocks > 0)) {
							//if the next is a parenthesys closure
							//don't put an space in between
							/*for (int j = i; j < code.length(); j++) {
								if (!invalid(code.charAt(j))) {
									if (code.charAt(j) != ')') {
										pcode += ' ';
										break;
									}
								}
							}*/
							pcode += ' ';
						}
					}
				}
			} else {
				pcode += car;
			}
		}
		return pcode;
	}	

	/**
	 * From here on there's a set of service functions
	 * used mostly during sepatate instruction
	 * evaluation.
	 * 
	 */
	
	public Functions getFunctions() {
		return functions;
	}

	public void setFunctions(Functions functions) {
		this.functions = functions;
	}

	public Variables getGlobal() {
		return global;
	}

	public void setGlobal(Variables global) {
		this.global = global;
	}

	/**
	 * The following three methods
	 * are the core of the stack.
	 * This allows functions to be
	 * recursive.
	 */
	
	public synchronized void setTemp(Variables vars) {
		temp.push(vars);
	}
	
	public synchronized Variables setTemp() {
		return temp.pop();
	}	
	
	public Variables getTemp() {
		return temp.peek();
	}

	/**
	 * These functions deal with variables.
	 * 
	 * In case of doubt, just use recurseVar.
	 */
	
	public Variable getVar(String name) {
		//Test local variables
		Variable var = getTemp().get(name);
		if (var == null) {
			//Test global variables
			var = getGlobal().get(name);
			if (var == null) {
				return null;
			} 
		}
		return var;
	}
	
	public Variable recurseVar(String name) {
		Variable var = getVar(name), aux = null;
		while (var != null) {
			aux = var;
			var = getVar(var.ref.value.toString());
		}
		return aux;
	}

	public String[] fetchArgs(Arguments args) {
		String[] str = new String[args.countArgs()-1];
		Variable var;
		Argument arg;
		for (int i = 1; i < args.countArgs(); i++) {
			arg = args.argumentAt(i);
			var = recurseVar(arg.getValue());
			if (var == null) {
				str[i-1] = arg.getValue();
			} else {
				str[i-1] = var.ref.value.toString();
				/*if (str[i-1].length() == 0) {
					str[i-1] = var.ref.value.toString();
				}*/
			}
			if (str[i-1].length() > 0 && str[i-1].charAt(0) == '\"') {
				str[i-1] = str[i-1].substring(1, str[i-1].length()-1);
			}
		}
		return str;
	}
	
	public void fetchVars(Arguments args) {
		Variable var = null;
		
		for (int i = 1; i < args.countArgs(); i++) {
			var = recurseVar(args.argumentAt(i).getValue());
			if (var != null) {
				args.argumentAt(i).setVar(var);
			}
		}
		
	}
	
	public static void main(String[] args) {
		//Programa real
		if (args.length < 1) {
			System.out.println("Usage: Program file");
			return;
		}
		Program  p = new Program();
		p.executeFile(args[0]);
	}

	public References getReferences() {
		return references;
	}

	public Macros getMacros() {
		return macros;
	}

	public boolean isDebug() {
		return DEBUG;
	}

	public void setDebug(boolean debug) {
		DEBUG = debug;
	}

	public RequestInput getInput() {
		if (input == null) {
			input = new RequestInput();
		}
		return input;
	}

	public void setInput(RequestInput input) {
		this.input = input;
	}

	public Types getTypes() {
		return types;
	}

	public void setTypes(Types types) {
		this.types = types;
	}
}
